@@ -1,62 +1,58 @@
 // -*- c-basic-offset:4; indent-tabs-mode:nil -*-
-/* This file is part of the KDE libraries
-   Copyright (C) 2000 David Faure <faure@kde.org>
-   Copyright (C) 2003 Alexander Kellett <lypanov@kde.org>
-   Copyright (C) 2008 Norbert Frese <nf2@scheinwelt.at>
+/*
+    This file is part of the KDE libraries
+    SPDX-FileCopyrightText: 2000 David Faure <faure@kde.org>
+    SPDX-FileCopyrightText: 2003 Alexander Kellett <lypanov@kde.org>
+    SPDX-FileCopyrightText: 2008 Norbert Frese <nf2@scheinwelt.at>
 
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
-   License version 2 as published by the Free Software Foundation.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public License
-   along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-only
 */
 
 #include "kbookmarkmanager.h"
-
+#include "kbookmarkdialog.h"
+#include "kbookmarkimporter.h"
+#include "kbookmarkmenu.h"
+#include "kbookmarkmenu_p.h"
 #include "kbookmarks_debug.h"
+#ifndef KBOOKMARKS_NO_DBUS
+#include "kbookmarkmanageradaptor_p.h"
+#endif
+
 #include <QDir>
 #include <QFile>
 #include <QFileInfo>
 #include <QProcess>
 #include <QRegularExpression>
-#include <QTextStream>
 #include <QTextCodec>
+#include <QTextStream>
 #ifndef KBOOKMARKS_NO_DBUS
 #include <QDBusConnection>
 #include <QDBusMessage>
 #endif
-#include <QMessageBox>
 #include <QApplication>
+#include <QMessageBox>
 #include <QReadWriteLock>
 #include <QThread>
 
-#include <kbackup.h>
-#include <kconfig.h>
-#include <kconfiggroup.h>
+#include <KBackup>
+#include <KConfig>
+#include <KConfigGroup>
 #include <KDirWatch>
-#include <qsavefile.h>
-#include <qstandardpaths.h>
+#include <QSaveFile>
+#include <QStandardPaths>
 
-#include "kbookmarkmenu.h"
-#include "kbookmarkmenu_p.h"
-#include "kbookmarkimporter.h"
-#include "kbookmarkdialog.h"
-#ifndef KBOOKMARKS_NO_DBUS
-#include "kbookmarkmanageradaptor_p.h"
-#endif
-
-namespace {
-namespace Strings {
-QString bookmarkChangeNotifyInterface() { return QStringLiteral("org.kde.KIO.KBookmarkManager"); }
-QString piData() { return QStringLiteral("version=\"1.0\" encoding=\"UTF-8\""); }
+namespace
+{
+namespace Strings
+{
+QString bookmarkChangeNotifyInterface()
+{
+    return QStringLiteral("org.kde.KIO.KBookmarkManager");
+}
+QString piData()
+{
+    return QStringLiteral("version=\"1.0\" encoding=\"UTF-8\"");
+}
 }
 }
 
@@ -68,7 +64,8 @@ public:
     {
         cleanup();
     }
-    void cleanup() {
+    void cleanup()
+    {
         QList<KBookmarkManager *> copy = *this;
         qDeleteAll(copy); // auto-delete functionality
         clear();
@@ -79,7 +76,8 @@ public:
 
 Q_GLOBAL_STATIC(KBookmarkManagerList, s_pSelf)
 
-static void deleteManagers() {
+static void deleteManagers()
+{
     if (s_pSelf.exists()) {
         s_pSelf->cleanup();
     }
@@ -94,7 +92,10 @@ KBookmarkManagerList::KBookmarkManagerList()
 class KBookmarkMap : private KBookmarkGroupTraverser
 {
 public:
-    KBookmarkMap() : m_mapNeedsUpdate(true) {}
+    KBookmarkMap()
+        : m_mapNeedsUpdate(true)
+    {
+    }
     void setNeedsUpdate()
     {
         m_mapNeedsUpdate = true;
@@ -104,6 +105,7 @@ public:
     {
         return m_bk_map.value(url);
     }
+
 private:
     void visit(const KBookmark &) override;
     void visitEnter(const KBookmarkGroup &) override
@@ -114,6 +116,7 @@ private:
     {
         ;
     }
+
 private:
     typedef QList<KBookmark> KBookmarkList;
     QMap<QString, KBookmarkList> m_bk_map;
@@ -154,7 +157,8 @@ public:
         , m_browserEditor(false)
         , m_typeExternal(false)
         , m_dirWatch(nullptr)
-    {}
+    {
+    }
 
     ~KBookmarkManagerPrivate()
     {
@@ -174,7 +178,7 @@ public:
     QString m_editorCaption;
 
     bool m_typeExternal;
-    KDirWatch *m_dirWatch;   // for external bookmark files
+    KDirWatch *m_dirWatch; // for external bookmark files
 
     KBookmarkMap m_map;
 };
@@ -184,8 +188,7 @@ public:
 
 static KBookmarkManager *lookupExisting(const QString &bookmarksFile)
 {
-    for (KBookmarkManagerList::ConstIterator bmit = s_pSelf()->constBegin(), bmend = s_pSelf()->constEnd();
-            bmit != bmend; ++bmit) {
+    for (KBookmarkManagerList::ConstIterator bmit = s_pSelf()->constBegin(), bmend = s_pSelf()->constEnd(); bmit != bmend; ++bmit) {
         if ((*bmit)->path() == bookmarksFile) {
             return *bmit;
         }
@@ -259,10 +262,11 @@ static QDomElement createXbelTopLevelElement(QDomDocument &doc)
 KBookmarkManager::KBookmarkManager(const QString &bookmarksFile, const QString &dbusObjectName)
     : d(new KBookmarkManagerPrivate(false, dbusObjectName))
 {
-    if (dbusObjectName.isNull()) // get dbusObjectName from file
+    if (dbusObjectName.isNull()) { // get dbusObjectName from file
         if (QFile::exists(d->m_bookmarksFile)) {
-            parse();    //sets d->m_dbusObjectName
+            parse(); // sets d->m_dbusObjectName
         }
+    }
 
     init(QLatin1String("/KBookmarkManager/") + d->m_dbusObjectName);
 
@@ -298,12 +302,9 @@ KBookmarkManager::KBookmarkManager(const QString &bookmarksFile)
     // start KDirWatch
     d->m_dirWatch = new KDirWatch;
     d->m_dirWatch->addFile(d->m_bookmarksFile);
-    QObject::connect(d->m_dirWatch, &KDirWatch::dirty,
-            this, &KBookmarkManager::slotFileChanged);
-    QObject::connect(d->m_dirWatch, &KDirWatch::created,
-            this, &KBookmarkManager::slotFileChanged);
-    QObject::connect(d->m_dirWatch, &KDirWatch::deleted,
-            this, &KBookmarkManager::slotFileChanged);
+    QObject::connect(d->m_dirWatch, &KDirWatch::dirty, this, &KBookmarkManager::slotFileChanged);
+    QObject::connect(d->m_dirWatch, &KDirWatch::created, this, &KBookmarkManager::slotFileChanged);
+    QObject::connect(d->m_dirWatch, &KDirWatch::deleted, this, &KBookmarkManager::slotFileChanged);
 
     // qCDebug(KBOOKMARKS_LOG) << "starting KDirWatch for" << d->m_bookmarksFile;
 }
@@ -326,28 +327,37 @@ void KBookmarkManager::init(const QString &dbusPath)
         new KBookmarkManagerAdaptor(this);
         QDBusConnection::sessionBus().registerObject(dbusPath, this);
 
-        QDBusConnection::sessionBus().connect(QString(), dbusPath, Strings::bookmarkChangeNotifyInterface(),
-                                              QStringLiteral("bookmarksChanged"), this, SLOT(notifyChanged(QString,QDBusMessage)));
-        QDBusConnection::sessionBus().connect(QString(), dbusPath, Strings::bookmarkChangeNotifyInterface(),
-                                              QStringLiteral("bookmarkConfigChanged"), this, SLOT(notifyConfigChanged()));
+        QDBusConnection::sessionBus().connect(QString(),
+                                              dbusPath,
+                                              Strings::bookmarkChangeNotifyInterface(),
+                                              QStringLiteral("bookmarksChanged"),
+                                              this,
+                                              SLOT(notifyChanged(QString, QDBusMessage)));
+        QDBusConnection::sessionBus()
+            .connect(QString(), dbusPath, Strings::bookmarkChangeNotifyInterface(), QStringLiteral("bookmarkConfigChanged"), this, SLOT(notifyConfigChanged()));
     }
 #endif
 }
 
 void KBookmarkManager::startKEditBookmarks(const QStringList &args)
 {
-    bool success = QProcess::startDetached(QStringLiteral(KEDITBOOKMARKS_BINARY), args);
+    bool success = false;
+    const QString exec = QStandardPaths::findExecutable(QStringLiteral(KEDITBOOKMARKS_BINARY));
+    if (!exec.isEmpty()) {
+        success = QProcess::startDetached(exec, args);
+    }
 
     if (!success) {
-        QString err = tr("Cannot launch keditbookmarks.\n\n"
-                         "Most likely you do not have keditbookmarks currently installed");
+        QString err =
+            tr("Cannot launch keditbookmarks.\n\n"
+               "Most likely you do not have keditbookmarks currently installed");
 
         if (d->m_dialogAllowed && qobject_cast<QApplication *>(qApp) && QThread::currentThread() == qApp->thread()) {
             QMessageBox::warning(QApplication::activeWindow(), QApplication::applicationName(), err);
         }
 
         qCWarning(KBOOKMARKS_LOG) << QStringLiteral("Failed to start keditbookmarks");
-        emit this->error(err);
+        Q_EMIT this->error(err);
     }
 }
 
@@ -359,7 +369,7 @@ void KBookmarkManager::slotFileChanged(const QString &path)
         parse();
         // Tell our GUI
         // (emit where group is "" to directly mark the root menu as dirty)
-        emit changed(QLatin1String(""), QString());
+        Q_EMIT changed(QLatin1String(""), QString());
     }
 }
 
@@ -368,8 +378,6 @@ KBookmarkManager::~KBookmarkManager()
     if (!s_pSelf.isDestroyed()) {
         s_pSelf()->removeAll(this);
     }
-
-    delete d;
 }
 
 bool KBookmarkManager::autoErrorHandlingEnabled() const
@@ -403,7 +411,9 @@ void KBookmarkManager::parse() const
     // qCDebug(KBOOKMARKS_LOG) << "KBookmarkManager::parse " << d->m_bookmarksFile;
     QFile file(d->m_bookmarksFile);
     if (!file.open(QIODevice::ReadOnly)) {
-        qCWarning(KBOOKMARKS_LOG) << "Can't open " << d->m_bookmarksFile;
+        qCWarning(KBOOKMARKS_LOG) << "Can't open" << d->m_bookmarksFile;
+        d->m_doc = QDomDocument(QStringLiteral("xbel"));
+        createXbelTopLevelElement(d->m_doc);
         return;
     }
     d->m_doc = QDomDocument(QStringLiteral("xbel"));
@@ -478,7 +488,10 @@ bool KBookmarkManager::saveAs(const QString &filename, bool toolbarCache) const
     if (file.open(QIODevice::WriteOnly)) {
         KBackup::simpleBackupFile(file.fileName(), QString(), QStringLiteral(".bak"));
         QTextStream stream(&file);
+        // In Qt6 it's UTF-8 by default
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
         stream.setCodec(QTextCodec::codecForName("UTF-8"));
+#endif
         stream << internalDocument().toString();
         stream.flush();
         if (file.commit()) {
@@ -491,14 +504,16 @@ bool KBookmarkManager::saveAs(const QString &filename, bool toolbarCache) const
         QString err = tr("Unable to save bookmarks in %1. Reported error was: %2. "
                          "This error message will only be shown once. The cause "
                          "of the error needs to be fixed as quickly as possible, "
-                         "which is most likely a full hard drive.").arg(filename, file.errorString());
+                         "which is most likely a full hard drive.")
+                          .arg(filename, file.errorString());
 
         if (d->m_dialogAllowed && qobject_cast<QApplication *>(qApp) && QThread::currentThread() == qApp->thread()) {
             QMessageBox::critical(QApplication::activeWindow(), QApplication::applicationName(), err);
         }
 
-        qCCritical(KBOOKMARKS_LOG) << QStringLiteral("Unable to save bookmarks in %1. File reported the following error-code: %2.").arg(filename).arg(file.error());
-        emit const_cast<KBookmarkManager *>(this)->error(err);
+        qCCritical(KBOOKMARKS_LOG)
+            << QStringLiteral("Unable to save bookmarks in %1. File reported the following error-code: %2.").arg(filename).arg(file.error());
+        Q_EMIT const_cast<KBookmarkManager *>(this)->error(err);
     }
     hadSaveError = true;
     return false;
@@ -523,9 +538,7 @@ KBookmarkGroup KBookmarkManager::toolbar()
         const QString cacheFilename = d->m_bookmarksFile + QLatin1String(".tbcache");
         QFileInfo bmInfo(d->m_bookmarksFile);
         QFileInfo cacheInfo(cacheFilename);
-        if (d->m_toolbarDoc.isNull() &&
-                QFile::exists(cacheFilename) &&
-                bmInfo.lastModified() < cacheInfo.lastModified()) {
+        if (d->m_toolbarDoc.isNull() && QFile::exists(cacheFilename) && bmInfo.lastModified() < cacheInfo.lastModified()) {
             // qCDebug(KBOOKMARKS_LOG) << "KBookmarkManager::toolbar reading file";
             QFile file(cacheFilename);
 
@@ -560,18 +573,16 @@ KBookmark KBookmarkManager::findByAddress(const QString &address)
     // qCDebug(KBOOKMARKS_LOG) << "KBookmarkManager::findByAddress " << address;
     KBookmark result = root();
     // The address is something like /5/10/2+
-#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
-    const QStringList addresses = address.split(QRegularExpression(QStringLiteral("[/+]")), QString::SkipEmptyParts);
-#else
-    const QStringList addresses = address.split(QRegularExpression(QStringLiteral("[/+]")), Qt::SkipEmptyParts);
-#endif
+    static const QRegularExpression separator(QStringLiteral("[/+]"));
+    const QStringList addresses = address.split(separator, Qt::SkipEmptyParts);
     // qCWarning(KBOOKMARKS_LOG) << addresses.join(",");
     for (QStringList::const_iterator it = addresses.begin(); it != addresses.end();) {
         bool append = ((*it) == QLatin1String("+"));
         uint number = (*it).toUInt();
         Q_ASSERT(result.isGroup());
         KBookmarkGroup group = result.toGroup();
-        KBookmark bk = group.first(), lbk = bk; // last non-null bookmark
+        KBookmark bk = group.first();
+        KBookmark lbk = bk; // last non-null bookmark
         for (uint i = 0; ((i < number) || append) && !bk.isNull(); ++i) {
             lbk = bk;
             bk = group.next(bk);
@@ -595,23 +606,23 @@ void KBookmarkManager::emitChanged()
 
 void KBookmarkManager::emitChanged(const KBookmarkGroup &group)
 {
-    (void) save(); // KDE5 TODO: emitChanged should return a bool? Maybe rename it to saveAndEmitChanged?
+    (void)save(); // KDE5 TODO: emitChanged should return a bool? Maybe rename it to saveAndEmitChanged?
 
     // Tell the other processes too
     // qCDebug(KBOOKMARKS_LOG) << "KBookmarkManager::emitChanged : broadcasting change " << group.address();
 
-    emit bookmarksChanged(group.address());
+    Q_EMIT bookmarksChanged(group.address());
 
     // We do get our own broadcast, so no need for this anymore
-    //emit changed( group );
+    // emit changed( group );
 }
 
 void KBookmarkManager::emitConfigChanged()
 {
-    emit bookmarkConfigChanged();
+    Q_EMIT bookmarkConfigChanged();
 }
 
-void KBookmarkManager::notifyCompleteChange(const QString &caller)   // DBUS call
+void KBookmarkManager::notifyCompleteChange(const QString &caller) // DBUS call
 {
     if (!d->m_update) {
         return;
@@ -623,7 +634,7 @@ void KBookmarkManager::notifyCompleteChange(const QString &caller)   // DBUS cal
     parse();
     // Tell our GUI
     // (emit where group is "" to directly mark the root menu as dirty)
-    emit changed(QLatin1String(""), caller);
+    Q_EMIT changed(QLatin1String(""), caller);
 }
 
 void KBookmarkManager::notifyConfigChanged() // DBUS call
@@ -631,11 +642,11 @@ void KBookmarkManager::notifyConfigChanged() // DBUS call
     // qCDebug(KBOOKMARKS_LOG) << "reloaded bookmark config!";
     KBookmarkSettings::self()->readSettings();
     parse(); // reload, and thusly recreate the menus
-    emit configChanged();
+    Q_EMIT configChanged();
 }
 
 #ifndef KBOOKMARKS_NO_DBUS
-void KBookmarkManager::notifyChanged(const QString &groupAddress, const QDBusMessage &msg)   // DBUS call
+void KBookmarkManager::notifyChanged(const QString &groupAddress, const QDBusMessage &msg) // DBUS call
 {
     // qCDebug(KBOOKMARKS_LOG) << "KBookmarkManager::notifyChanged ( "<<groupAddress<<")";
     if (!d->m_update) {
@@ -649,9 +660,9 @@ void KBookmarkManager::notifyChanged(const QString &groupAddress, const QDBusMes
     }
 
     // qCDebug(KBOOKMARKS_LOG) << "KBookmarkManager::notifyChanged " << groupAddress;
-    //KBookmarkGroup group = findByAddress( groupAddress ).toGroup();
-    //Q_ASSERT(!group.isNull());
-    emit changed(groupAddress, QString());
+    // KBookmarkGroup group = findByAddress( groupAddress ).toGroup();
+    // Q_ASSERT(!group.isNull());
+    Q_EMIT changed(groupAddress, QString());
 }
 #endif
 
@@ -689,8 +700,7 @@ void KBookmarkManager::slotEditBookmarksAtAddress(const QString &address)
     if (!d->m_dbusObjectName.isEmpty()) {
         args << QStringLiteral("--dbusObjectName") << d->m_dbusObjectName;
     }
-    args << QStringLiteral("--address") << address
-         << d->m_bookmarksFile;
+    args << QStringLiteral("--address") << address << d->m_bookmarksFile;
     startKEditBookmarks(args);
 }
 
@@ -703,20 +713,18 @@ bool KBookmarkManager::updateAccessMetadata(const QString &url)
         return false;
     }
 
-    for (QList<KBookmark>::iterator it = list.begin();
-            it != list.end(); ++it) {
+    for (QList<KBookmark>::iterator it = list.begin(); it != list.end(); ++it) {
         (*it).updateAccessMetadata();
     }
 
     return true;
 }
 
-void KBookmarkManager::updateFavicon(const QString &url, const QString &/*faviconurl*/)
+void KBookmarkManager::updateFavicon(const QString &url, const QString & /*faviconurl*/)
 {
     d->m_map.update(this);
     QList<KBookmark> list = d->m_map.find(url);
-    for (QList<KBookmark>::iterator it = list.begin();
-            it != list.end(); ++it) {
+    for (QList<KBookmark>::iterator it = list.begin(); it != list.end(); ++it) {
         // TODO - update favicon data based on faviconurl
         //        but only when the previously used icon
         //        isn't a manually set one.
@@ -725,7 +733,8 @@ void KBookmarkManager::updateFavicon(const QString &url, const QString &/*favico
 
 KBookmarkManager *KBookmarkManager::userBookmarksManager()
 {
-    const QString bookmarksFile = QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + QLatin1Char('/') + QLatin1String("konqueror/bookmarks.xml");
+    const QString bookmarksFile =
+        QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + QLatin1String("/konqueror/bookmarks.xml");
     KBookmarkManager *bookmarkManager = KBookmarkManager::managerForFile(bookmarksFile, QStringLiteral("konqueror"));
     QString caption = QGuiApplication::applicationDisplayName();
     if (caption.isEmpty()) {
